function math.clamp(input, min_val, max_val)
	if input < min_val then
		return min_val
	elseif input > max_val then
		return max_val
	end
	return input
end

local lg = love.graphics
local canHazCanvas = lg.isSupported 'canvas'

if canHazCanvas then 
	canvas = lg.newCanvas(1024, 1024)
	lg.setCanvas(canvas)
else
	require 'run'
	lg.clear()
end

love.keyboard.setKeyRepeat(.25, .025)

love.load = function()

	templ = setmetatable({}, {__index = function() return {} end})

--	l.clear()

	screenW, screenH = 800, 600
	
	if canHazCanvas then 
		canvas = lg.newCanvas(1024, 1024)
		lg.setCanvas(canvas)
	else
		require 'run'
		lg.clear()
	end

	Sounds = {}
	
	mode = true
	
	time, oldTime = 1, 1
	bpm = 120
	secPerBeat = 2
	
	Sounds[true] = (love.filesystem.load 'notes.lua' 'sine').sources
	Sounds[false] = (love.filesystem.load 'notes.lua' 'kick.ogg').sources
	
	field = {}
	field.len = 128
	
	field.tileW, field.tileH = 6, 6
	field.w, field.h = 128, 88
	field.offX, field.offY = 36, 36
	
	lg.setLine(1, 'rough')
	
	for i = 0, field.w+1 do
	
		field[i] = {}
		
		for ii = 1, field.h do
		
			field[i][ii] = {}
			if not (i == 0) then
				lg.rectangle('line', (i-1)*field.tileW+field.offX, (ii-1)*field.tileH+field.offY, field.tileW, field.tileH)
			end
			
		end
		
	end
	
	local f = lg.newFont(12)
	
	lg.setFont(lg.newFont(9))
	
	for i, v in ipairs(require 'notesList') do
	
		lg.printf(v, 0, (i-1)*field.tileH+field.offY, field.offX-2-((i+1)%2)*15, 'right')
		
	end
	
	if canHazCanvas then
		lg.setCanvas()
	else
		lg.present()
	end
	
	lg.setFont(f)
	
end


love.update = function(dt)

	local x, y = love.mouse.getPosition()

	fieldX, fieldY = math.floor((x-field.offX)/field.tileW)+1, math.floor((y-field.offY)/field.tileH)+1

	local temp

	if pause then return end

	time = time+dt
	
	currentBeat, oldBeat = math.floor(time*secPerBeat), math.floor(oldTime*secPerBeat)
	
	if currentBeat > field.len then
	
		currentBeat = currentBeat%field.len
		
	end
	
	if oldBeat > field.len then
	
		oldBeat = oldBeat%field.len
		
	end
	
	if not (currentBeat == oldBeat) then
	
		temp = Sounds[mode]
	
		for i = 1, 88 do
			
			local v = field[oldBeat][i]
			if (v[3] and v[1]) or (v[1] and not v[2]) then
			
				
				temp[89-i]:stop()
				
			end
			
			if (field[currentBeat] or templ)[i][1] and not (field[currentBeat] or templ)[i][3] then
			
				temp[89-i]:play()
				
			end
			
		end
		
		if canHazCanvas then
			lg.setCanvas(canvas)
		else
			lg.present()
		end
		lg.setColor(0,0,0)
		lg.rectangle('fill', 0, screenH-field.offY+1, screenW, field.offY)
		lg.rectangle('fill', field.offX-field.tileW, 0, screenW, field.offY)
		lg.setColor(255,0,0)
		lg.rectangle('fill', (currentBeat-1)*field.tileW+field.offX, 0, field.tileW, field.offY)
		lg.rectangle('fill', (currentBeat-1)*field.tileW+field.offX, screenH-field.offY+1, field.tileW, field.offY)
		if canHazCanvas then
			lg.setCanvas()
		else
			lg.present()
		end
		lg.setColor(255,255,255)
		
	end
	
	oldTime = time

end


love.draw = function()
	
	lg.setBlendMode 'premultiplied'

	lg.draw(canvas, 0, 0)
	
	lg.setBlendMode 'alpha'
	
	lg.print(bpm, 0, 0)
	lg.print((mode and "Sine" or "Kick.ogg"), 0, 12)
	_ = pause and lg.print('PAUSED', 0, 24)
	
	love.graphics.setColor(0, 0, 192, 128)
	
	lg.rectangle('fill', math.max((fieldX-1)*field.tileW+field.offX, field.offX), field.offY, field.tileW-1, screenH-field.offY*2)
	
	lg.rectangle('fill', field.offX, math.clamp((fieldY-1)*field.tileH+field.offY+1, field.offY, screenH-field.offY-field.tileH+1), screenW, field.tileH-1)
	
	lg.setColor(255,255,255)
	
end


love.mousepressed = function(x, y, b)

	if b == 'l' then

		--local fieldX, fieldY = math.floor((x-field.offX)/field.tileW)+1, math.floor((y-field.offY)/field.tileH)+1
		
		if not (fieldX <= field.w and fieldX > 0 and fieldY <= field.h and fieldY > 0) then
		
			return
			
		end
		
		
		if not field[fieldX][fieldY][1] then
		
			field[fieldX][fieldY] = {true, false, false}
			
		else
		
			field[fieldX][fieldY] = {}
			
		end
	
		if field[fieldX][fieldY][1] then
		
			lg.setColor(255,255,255)
			
		else
		
			lg.setColor(0,0,0)
			
		end
	
		if canHazCanvas then
			lg.setCanvas(canvas)
		else
			lg.present()
		end
		lg.rectangle('fill', (fieldX-1)*field.tileW+field.offX, (fieldY-1)*field.tileH+field.offY+1, field.tileW-1, field.tileH-1)
		if canHazCanvas then
			lg.setCanvas()
		else
			lg.present()
		end
			
		lg.setColor(255,255,255)
	
	elseif b == 'r' then
	
		if not (fieldX <= field.w and fieldX > 0 and fieldY <= field.h and fieldY > 0) then
		
			return
			
		end
		
		if not field[fieldX][fieldY][1] or not field[fieldX][fieldY][2] then
		
			field[fieldX][fieldY] = {true, true, false}
			
			lg.setColor(0, 255, 0)
			
		else
			
			field[fieldX][fieldY][3] = not field[fieldX][fieldY][3]
			
			local t = field[fieldX][fieldY][3]
			lg.setColor(t and 255 or 0, t and 0 or 255, 0)
			
		end
		
		if canHazCanvas then
			lg.setCanvas(canvas)
		else
			lg.present()
		end
		lg.rectangle('fill', (fieldX-1)*field.tileW+field.offX, (fieldY-1)*field.tileH+field.offY+1, field.tileW-1, field.tileH-1)
		if canHazCanvas then
			lg.setCanvas()
		else
			lg.present()
		end
		
		lg.setColor(255,255,255)
		
	end
	
end


love.keypressed = function(k)

	if k == ' ' then
	
		pause = not pause
		love.audio.stop()
		return

	elseif k == 'm' then
	
		mode = not mode
		return
		
	elseif k == 'up' then
	
		bpm = bpm + 1
		secPerBeat = bpm/60
		BPS = 1/secPerBeat
		if not canHazCanvas then
			lg.present()
			lg.setColor(0,0,0)
			lg.rectangle('fill', 0, 0, field.offX, 12)
			lg.setColor(255,255,255)
			lg.print(bpm, 0, 0)
			lg.present()
		end
		time = 0
		return
		
	elseif k == 'down' then
	
		bpm = bpm - 1
		secPerBeat = bpm/60
		BPS = 1/secPerBeat
		if not canHazCanvas then
			lg.present()
			lg.setColor(0,0,0)
			lg.rectangle('fill', 0, 0, field.offX, 12)
			lg.setColor(255,255,255)
			lg.print(bpm, 0, 0)
			lg.present()
		end
		time = 0
		return
		
	end
	
end